@@ -1,11 +1,19 @@
-## ----include=FALSE-------------------------------------------------------
+## ----setup, include=FALSE-----------------------------------------------------
+library(knitr)
+knit_hooks$set(inline = function(x) {
+  if (is.numeric(x)) return(knitr:::format_sci(x, 'latex'))
+  highr::hi_latex(x)
+})
+
+## ----include=FALSE------------------------------------------------------------
 library(knitr)
 
 ## ----exampleload,eval=TRUE,include=FALSE----------------------------
 library("irace")
-load("examples.Rdata")
-load("irace-acotsp.Rdata")
-load("log-ablation.Rdata")
+load("examples.Rdata") # loads "experiment" and "output"
+iraceResults <- irace::read_logfile(system.file(package="irace", "exdata", "irace-acotsp.Rdata", mustWork=TRUE))
+log_ablation_file <- system.file(package="irace", "exdata", "log-ablation.Rdata", mustWork = TRUE)
+load(log_ablation_file)
 options(width = 70)
 
 ## ----R_irace_install, prompt=FALSE, eval=FALSE----------------------
@@ -34,7 +42,7 @@ options(width = 70)
 
 ## ----windows_irace_help ,eval=FALSE, prompt=FALSE-------------------
 #  library("irace")
-#  irace.cmdline("--help")
+#  irace_cmdline("--help")
 
 ## ----irace_R_check, eval=FALSE, prompt=FALSE------------------------
 #  library("irace")
@@ -45,25 +53,25 @@ options(width = 70)
 ## ----irace_R_exe, eval=FALSE, prompt=FALSE--------------------------
 #  library("irace")
 #  # Go to the directory containing the scenario files
-#  setwd("~/tuning")
+#  setwd("./tuning")
 #  scenario <- readScenario(filename = "scenario.txt",
 #                           scenario = defaultScenario())
-#  irace.main(scenario = scenario)
+#  irace_main(scenario = scenario)
 
 ## ----runexample2,prompt=FALSE,eval=FALSE----------------------------
 #  library("irace")
-#  setwd("~/tuning/")
-#  irace.cmdline()
+#  setwd("./tuning/")
+#  irace_cmdline()
 
 ## ----readParameters,prompt=FALSE, eval=FALSE------------------------
 #  parameters <- readParameters(file = "parameters.txt")
 
 ## ----parametersetup,eval=TRUE,include=FALSE-------------------------
 # Setup example
-parameters <- irace::readParameters(text='
+parameters <- readParameters(text='
 algorithm       "--"                 c		(as,mmas,eas,ras,acs)
-ants            "--ants "            i	  	(5, 100)
-q0              "--q0 "              r  	(0.0, 1.0) 		| algorithm %in% c("acs")
+ants            "--ants "            i	        (5, 100)
+q0              "--q0 "              r          (0.0, 1.0)              | algorithm %in% c("acs")
 ')
 
 ## ----parameterlist,eval=TRUE, prompt=TRUE, size='normalsize', comment=""----
@@ -75,9 +83,9 @@ str(parameters, vec.len = 10)
 ## ----experimentlist,eval=TRUE,size='normalsize', prompt=TRUE,  comment=""----
 print(experiment)
 
-## ----targetEvaluator, prompt=FALSE, eval=FALSE----------------------
-#  targetEvaluator(experiment, num.configurations, all.conf.id,
-#                  scenario, target.runner.call)
+## ----targetEvaluator, eval=FALSE------------------------------------
+#  targetEvaluator(experiment, num_configurations, all_conf_id, scenario,
+#                  target_runner_call)
 
 ## ----instance1, prompt=FALSE, eval=FALSE----------------------------
 #  scenario$instances <- c("rosenbrock_20 --function=12 --nvar 20",
@@ -86,54 +94,56 @@ print(experiment)
 #                          "rastrigin_40 --function=15 --nvar 30")
 
 ## ----repairEx,prompt=FALSE, eval=FALSE------------------------------
-#  repairConfiguration = function (configuration, parameters, digits)
+#  repairConfiguration = function(configuration, parameters)
 #  {
-#    isreal <- parameters$type[colnames(configuration)] %in% "r"
-#    configuration[isreal] <- configuration[isreal] / sum(configuration[isreal])
+#    isreal <- names(which(parameters$types[colnames(configuration)] == "r"))
+#    # This ignores 'digits'
+#    c_real <- unlist(configuration[isreal])
+#    c_real <- c_real / sum(c_real)
+#    configuration[isreal] <- c_real
 #    return(configuration)
 #  }
 
 ## ----repairEx2,prompt=FALSE, eval=FALSE-----------------------------
-#  repairConfiguration = function (configuration, parameters, digits)
+#  repairConfiguration = function(configuration, parameters)
 #  {
 #   columns <- c("p1","p2","p3")
 #   # cat("Before"); print(configuration)
-#   configuration[columns] <- sort(configuration[columns])
+#   configuration[columns] <- sort(unlist(configuration[columns], use.names=FALSE))
 #   # cat("After"); print(configuration)
 #   return(configuration)
 #  }
 
 ## ----targetRunnerParallel,prompt=FALSE, eval=FALSE------------------
-#  targetRunnerParallel(experiments, exec.target.runner, scenario, target.runner)
+#  targetRunnerParallel(experiments, exec_target_runner, scenario, target_runner)
 
 ## ----targetRunnerParallel2,prompt=FALSE, eval=FALSE-----------------
-#  targetRunnerParallel <- function(experiments, exec.target.runner, scenario)
+#  targetRunnerParallel <- function(experiments, exec_target_runner, scenario,
+#                                   target_runner)
 #  {
-#    return (lapply(experiments, exec.target.runner, scenario = scenario,
-#                   target.runner = target.runner))
+#    lapply(experiments, exec_target_runner, scenario = scenario,
+#           target_runner = target_runner)
 #  }
 
 ## ----targetRunnerParallel3,prompt=FALSE, eval=TRUE------------------
    print(output)
 
 ## ----testing_r, prompt=FALSE, eval=FALSE----------------------------
-#  testing.main(logFile = "./irace.Rdata")
+#  testing_fromlog(logFile = "./irace.Rdata", testNbElites = 1)
 
 ## ----change_recover, prompt=TRUE, eval=FALSE------------------------
-#  load ("~/tuning/irace.Rdata")
-#  new.path <- "~/experiments/tuning/instances/"
+#  iraceResults <- read_logfile("./tuning/irace.Rdata")
+#  new_path <- "./experiments/tuning/instances/"
 #  iraceResults$scenario$instances <-
-#     paste (new.path,
-#           basename(iraceResults$scenario$instances),
-#           sep="")
-#  save (iraceResults, file="~/tuning/irace.Rdata")
+#     paste0(new_path, basename(iraceResults$scenario$instances))
+#  save(iraceResults, file="./tuning/irace.Rdata")
 
 ## ----load_rdata, prompt=FALSE, eval=FALSE---------------------------
-#  load("irace-acotsp.Rdata")
+#  logfile <- system.file(package="irace", "exdata", "irace-acotsp.Rdata", mustWork=TRUE)
+#  iraceResults <- read_logfile(logfile)
 
 ## ----show_version, prompt=TRUE, eval=TRUE, comment=""---------------
-iraceResults$irace.version
-iraceResults[["irace.version"]]
+iraceResults$irace_version
 
 ## ----show_configurations, prompt=TRUE, eval=TRUE, comment=""--------
 head(iraceResults$allConfigurations)
@@ -142,7 +152,8 @@ head(iraceResults$allConfigurations)
 print(iraceResults$allElites)
 
 ## ----get_elites, prompt=TRUE, eval=TRUE, comment=""-----------------
-getFinalElites(logFile = "irace-acotsp.Rdata", n = 0)
+logfile <- system.file(package="irace", "exdata", "irace-acotsp.Rdata", mustWork=TRUE)
+getFinalElites(logfile, n = 0)
 
 ## ----show_iditelites, prompt=TRUE, eval=TRUE, comment=""------------
 print(iraceResults$iterationElites)
@@ -150,34 +161,48 @@ print(iraceResults$iterationElites)
 ## ----get_elite, prompt=TRUE, eval=TRUE, comment=""------------------
 last <- length(iraceResults$iterationElites)
 id <- iraceResults$iterationElites[last]
-getConfigurationById(logFile = "irace-acotsp.Rdata", ids = id)
+getConfigurationById(iraceResults, ids = id)
 
 ## ----get_experiments, prompt=TRUE, eval=TRUE, comment=""------------
 # As an example, we use the best configuration found
-best.config <- getFinalElites(iraceResults = iraceResults, n = 1)
-id <- best.config$.ID.
-# Obtain the configurations using the identifier
-# of the best configuration
-all.exp <- iraceResults$experiments[,as.character(id)]
-all.exp[!is.na(all.exp)]
+best_config <- getFinalElites(iraceResults, n = 1)
+best_id <- as.character(best_config$.ID.)
+# Obtain the results of the best configuration
+all_exp <- iraceResults$experiments[, best_id]
+# all_exp is a vector and names(all_exp) is the (instance,seed) index.
+all_exp
+# Obtain the results of the first and best configurations
+all_exp <- iraceResults$experiments[, c("1", best_id)]
+# all_exp is a matrix: colnames(all_exp) is configurationID and
+# rownames(all_exp) is the (instance,seed) index.
+all_exp
 
 ## ----get_instance_seed, prompt=TRUE, eval=TRUE, comment=""----------
-# As an example, we get seed and instance of the experiments
-# of the best candidate.
-# Get index of the instances
-pair.id <- names(all.exp[!is.na(all.exp)])
-index <- iraceResults$state$.irace$instancesList[pair.id,"instance"]
-# Obtain the instance names
-iraceResults$scenario$instances[index]
-# Get the seeds
-iraceResults$state$.irace$instancesList[index,"seed"]
+# As an example, we get instanceID, seeds and instances of the experiments
+# of the best configuration.
+# We could get the indexes of the instances on which at least one
+# configuration was executed:
+pair_index <- which(apply(!is.na(all_exp), 1L, any))
+# or the instances on which all configurations were executed:
+pair_index <- which(apply(!is.na(all_exp), 1L, all))
+# but in this example we get the indexes of the instances executed for
+# the best configuration.
+pair_index <- which(!is.na(all_exp[, best_id]))
+instanceID <- get_instanceID_seed_pairs(iraceResults)[["instanceID"]][pair_index]
+# or get the seeds
+get_instanceID_seed_pairs(iraceResults)[["seed"]][pair_index]
+# or obtain the actual instances.
+iraceResults$scenario$instances[instanceID]
+# If the instances are of atomic type (integers, floating-point numbers or
+# character strings), the above is similar to:
+get_instanceID_seed_pairs(iraceResults, index = pair_index, instances=TRUE)
 
 ## ----get_model, prompt=TRUE, eval=TRUE, comment=""------------------
 # As an example, we get the model probabilities for the
 # localsearch parameter.
 iraceResults$state$model["localsearch"]
 # The order of the probabilities corresponds to:
-iraceResults$parameters$domain$localsearch
+iraceResults$scenario$parameters$domains$localsearch
 
 ## ----get_test_exp, prompt=TRUE, eval=TRUE, comment=""---------------
 # Get the results of the testing
@@ -187,79 +212,59 @@ iraceResults$testing$experiments
 # Get the seeds used for testing
 iraceResults$testing$seeds
 
-## ----plot_test, fig.pos="tbp", fig.align="center", fig.height = 4, fig.width = 8, out.width='0.85\\textwidth', fig.cap="Boxplot of the testing results of the best configurations.", prompt=TRUE, eval=TRUE, comment=""----
+## ----wilcox_test,prompt=TRUE, eval=TRUE, comment=""-----------------
 results <- iraceResults$testing$experiments
 # Wilcoxon paired test
 conf <- gl(ncol(results), # number of configurations
            nrow(results), # number of instances
            labels = colnames(results))
 pairwise.wilcox.test (as.vector(results), conf, paired = TRUE, p.adj = "bonf")
-# Plot the results
-configurationsBoxplot (results, ylab = "Solution cost")
 
-## ----freq, fig.pos="tbp", fig.cap="Parameters sampling frequency.", out.width="\\textwidth",prompt=TRUE, eval=TRUE, comment=""----
-parameterFrequency(iraceResults$allConfigurations, iraceResults$parameters)
+## ----conc, prompt=TRUE, eval=TRUE, comment=""-----------------------
+irace:::concordance(iraceResults$testing$experiments)
 
-## ----parcord, fig.pos="tbp", fig.align="center", out.width="0.7\\textwidth", fig.cap="Parallel coordinate plots of the parameters of the configurations in the last two iterations of a run of \\irace.", prompt=FALSE, eval=TRUE, comment=""----
-# Get last iteration number
-last <- length(iraceResults$iterationElites)
-# Get configurations in the last two iterations
-conf <- getConfigurationByIteration(iraceResults = iraceResults,
-                                    iterations = c(last - 1, last))
-parallelCoordinatesPlot (conf, iraceResults$parameters,
-                         param_names = c("algorithm", "alpha", "beta", "rho", "q0"),
-                         hierarchy = FALSE)
-
-## ----trainEvo, fig.pos="tbp", fig.align="center", out.width='0.75\\textwidth', fig.cap="Training set performance of the best-so-far configuration over number of experiments.", prompt=FALSE, eval=TRUE, comment=""----
+## ----testEvo, fig.pos="tb", fig.align="center", out.width='0.7\\textwidth', fig.cap="Testing set performance of the best-so-far configuration over number of experiments. Label of each point is the configuration ID.", prompt=FALSE, eval=TRUE, comment=""----
+# Get summary data from the logfile.
+irs <- irace_summarise(iraceResults)
 # Get number of iterations
-iters <- unique(iraceResults$experimentLog[, "iteration"])
+iters <- irs$n_iterations
 # Get number of experiments (runs of target-runner) up to each iteration
-fes <- cumsum(sapply(iters, function(k)
-  sum(iraceResults$experimentLog[, "iteration"] == k)))
+fes <- cumsum(table(iraceResults$state$experiment_log[["iteration"]]))
 # Get the mean value of all experiments executed up to each iteration
 # for the best configuration of that iteration.
-values <- sapply(iters, function(k) {
-  instances <- as.character(
-    unique(iraceResults$experimentLog[
-                          iraceResults$experimentLog[, "iteration"] == k,
-                          "instance"]))
-  return(mean(iraceResults$experiments[
-                             instances,
-                             as.character(iraceResults$iterationElites[k])]))})
-plot(fes, values, type="s", xlab = "Number of runs of the target algorithm",
-     ylab = "Estimated mean value over whole training set")
-points(fes, values)
-
-## ----testEvo, fig.pos="tbp", fig.align="center", out.width='0.75\\textwidth', fig.cap="Testing set performance of the best-so-far configuration over number of experiments.", prompt=FALSE, eval=TRUE, comment=""----
-# Get number of iterations
-iters <- unique(iraceResults$experimentLog[, "iteration"])
-# Get number of experiments (runs of target-runner) up to each iteration
-fes <- cumsum(sapply(iters, function(k)
-  sum(iraceResults$experimentLog[, "iteration"] == k)))
-# Get the mean value of all experiments executed up to each iteration
-# for the best configuration of that iteration.
-values <- sapply(iters, function(k) 
-  mean(iraceResults$testing$experiments[
-                            , as.character(iraceResults$iterationElites[k])]))
+elites <- as.character(iraceResults$iterationElites)
+values <- colMeans(iraceResults$testing$experiments[, elites])
+stderr <- function(x) sqrt(var(x)/length(x))
+err <- apply(iraceResults$testing$experiments[, elites], 2L, stderr)
 plot(fes, values, type = "s",
      xlab = "Number of runs of the target algorithm",
-     ylab = "Mean value over testing set")
-points(fes, values)
+     ylab = "Mean value over testing set", ylim=c(23000000,23500000))
+points(fes, values, pch=19)
+arrows(fes, values - err, fes, values + err, length=0.05, angle=90, code=3)
+text(fes, values, elites, pos = 1)
 
 ## ----ablation, prompt=FALSE, eval=FALSE-----------------------------
-#  ablation(iraceLogFile = "irace.Rdata",
-#           src = 1, target = 60, pdf.file = "plot-ablation.pdf")
+#  ablog <- ablation("irace.Rdata", src = 1, target = 60)
+#  plotAblation(ablog)
 
-## ----testAb, fig.pos="htb!", fig.align="center", out.width="0.75\\textwidth", fig.cap="Example of plot generated by \\code{ablation()}.", prompt=FALSE, eval=TRUE, echo=FALSE----
-plotAblation(abLogFile = "log-ablation.Rdata")
+## ----testAb, fig.pos="htb!", fig.align="center", out.width="0.75\\textwidth", fig.cap="Example of plot generated by \\code{plotAblation()}.", prompt=FALSE, eval=TRUE, echo=FALSE----
+logfile <- system.file(package="irace", "exdata", "log-ablation.Rdata", mustWork=TRUE)
+plotAblation(logfile)
+
+## ----ablation_cmdline, prompt=FALSE, eval=TRUE,echo=FALSE, comment=""----
+ablation_cmdline("--help")
 
 ## ----postsel, prompt=FALSE, eval=FALSE------------------------------
 #  # Execute all elite configurations in the iterations
-#  psRace(iraceLogFile="irace.Rdata", elites=TRUE)
+#  psRace("irace.Rdata", max_experiments = 0.5, elites=TRUE)
 #  # Execute a set of configurations IDs providing budget
-#  psRace(iraceLogFile="irace.Rdata",
-#               conf.ids=c(34, 87, 102, 172, 293),
-#               max.experiments=500)
+#  psRace("irace.Rdata", conf_ids = c(34, 87, 102, 172, 293), max_experiments = 500)
+
+## ----targetCmdline, prompt=FALSE, eval=FALSE------------------------
+#  targetRunner="./real_target_runner.py"
+#  targetRunnerLauncher="python"
+#  targetCmdLine="-m {targetRunner} {configurationID} {instanceID}\
+#   --seed {seed} -i {instance} --cutoff {bound} {targetRunnerArgs}"
 
 ## ----faq3, eval=FALSE-----------------------------------------------
 #  library(Rmpi)
